@@ -12,6 +12,7 @@ from io import BytesIO
 import logging
 import base64
 from concurrent.futures import ThreadPoolExecutor, as_completed
+from scipy.spatial.distance import cosine
 
 # Global variables to store lazily loaded modules
 _deepface = None
@@ -606,10 +607,10 @@ def load_and_process_image_deepface(image_input, country=None):
                 enforce_detection=False,
                 align=True
             )
-            
+
             if face_objs and len(face_objs) > 0:
                 confidence = face_objs[0].get('confidence', 0)
-                
+
                 # print(f"Confidence: {confidence}")
 
                 # Check face frame size only if confidence is less than 1
@@ -622,14 +623,14 @@ def load_and_process_image_deepface(image_input, country=None):
                     elif country != 'SDN' and (facial_area['w'] < 80 or facial_area['h'] < 90):
                         print(f"Rejecting face at {angle} degrees due to small size: {facial_area['w']}x{facial_area['h']} (minimum 100x100)")
                         return None, None, 0
-                
+
                 # Immediately reject if confidence is below threshold
                 if confidence < 0.95:
                     # print(f"Rejecting face at {angle} degrees due to low confidence: {confidence:.3f}")
                     return None, None, 0
-                
+
                 return face_objs, img_to_process, confidence
-            
+
             # Clear memory if no face found
             del img_to_process
             return None, None, 0
@@ -669,7 +670,7 @@ def load_and_process_image_deepface(image_input, country=None):
             try:
                 biggest_face = max(face_objs, key=lambda face: face['facial_area']['w'] * face['facial_area']['h'])
                 facial_area = biggest_face['facial_area']
-                
+
                 # Double check size requirements for biggest face
                 if confidence < 1:
                     # print(f"Confidence less than 1: {confidence}")
@@ -679,14 +680,14 @@ def load_and_process_image_deepface(image_input, country=None):
                     elif country != 'SDN' and (facial_area['w'] < 80 or facial_area['h'] < 90):
                         print(f"Face validation failed: Face frame too small {facial_area['w']}x{facial_area['h']} (minimum 100x100)")
                         return [], []
-                    
+
                 x, y, w, h = facial_area['x'], facial_area['y'], facial_area['w'], facial_area['h']
-                
+
                 # Minimize memory usage during color conversion
                 image_rgb = cv2.cvtColor(processed_image, cv2.COLOR_BGR2RGB)
                 face_locations = [(y, x + w, y + h, x)]
                 face_encodings = face_recognition.face_encodings(image_rgb, face_locations)
-                
+
                 if face_encodings:
                     # print(f"Found face in original orientation with confidence {confidence}")
                     return face_locations, face_encodings
@@ -704,7 +705,7 @@ def load_and_process_image_deepface(image_input, country=None):
 
         with ThreadPoolExecutor(max_workers=3) as executor:
             futures = {
-                executor.submit(process_angle, image, angle): angle 
+                executor.submit(process_angle, image, angle): angle
                 for angle in angles
             }
 
@@ -732,23 +733,23 @@ def load_and_process_image_deepface(image_input, country=None):
         try:
             biggest_face = max(best_face_objs, key=lambda face: face['facial_area']['w'] * face['facial_area']['h'])
             facial_area = biggest_face['facial_area']
-            
+
             # Final size check for rotated face
             if confidence < 1:
                 if facial_area['w'] < 80 or facial_area['h'] < 90:
                     print(f"Face validation failed: Face frame too small {facial_area['w']}x{facial_area['h']} (minimum 100x100)")
                     return [], []
-                
+
             x, y, w, h = facial_area['x'], facial_area['y'], facial_area['w'], facial_area['h']
-            
+
             # Minimize memory during final processing
             image_rgb = cv2.cvtColor(best_image, cv2.COLOR_BGR2RGB)
             face_locations = [(y, x + w, y + h, x)]
             face_encodings = face_recognition.face_encodings(image_rgb, face_locations)
-            
+
             if face_encodings:
                 return face_locations, face_encodings
-            
+
             print("Failed to extract face encodings")
             return [], []
         finally:
@@ -763,6 +764,9 @@ def load_and_process_image_deepface(image_input, country=None):
         if 'image' in locals():
             del image
 
+
+
+
 def calculate_similarity(face_encoding1, face_encoding2):
     face_recognition = get_face_recognition()
     similarity_score = 1 - face_recognition.face_distance([face_encoding1], face_encoding2)[0]
@@ -903,3 +907,243 @@ def load_and_process_image_deepface_all_orientations(image_input):
         # Ensure main image is cleared
         if 'image' in locals():
             del image
+
+def get_facial_encodings_deepface_irq(image, country: str, model_name="ArcFace", detector_backend="retinaface"):
+    logging.info(f"Type: {type(image)} | Shape: {image.shape} | Dtype: {image.dtype}")
+
+    from deepface import DeepFace
+    import numpy as np
+
+    if image.ndim == 3 and image.shape[2] == 4:
+        # Convert RGBA to BGR (DeepFace expects BGR or RGB)
+        # OpenCV uses BGR by default
+        image = cv2.cvtColor(image, cv2.COLOR_BGRA2BGR)
+    elif image.ndim == 2:
+        # grayscale to BGR
+        image = cv2.cvtColor(image, cv2.COLOR_GRAY2BGR)
+
+    if image.dtype != np.uint8:
+        image = image.astype(np.uint8)
+
+
+    # DeepFace doesnt accepts 4 channel images (RGBA), so we convert to RGB or BGR
+    try:
+        logging.info(f"[DeepFace] Starting face extraction for {country} with model {model_name} and detector {detector_backend}")
+
+        face_objs = DeepFace.represent(
+            img_path=image,
+            model_name=model_name,
+            detector_backend=detector_backend,
+            enforce_detection=False,
+            align=True
+        )
+
+        logging.info(f"[DeepFace] Face extraction successful: {len(face_objs)} faces detected.")
+        # logging.info(f"[DeepFace] Face details: {face_objs}")
+    except Exception as e:
+        logging.error(f"[DeepFace] Face extraction failed: {e}")
+        return [],[]
+
+    if not face_objs:
+        logging.warning("No faces detected.")
+        return [], []
+
+    # Normalize to list
+    if isinstance(face_objs, dict):
+        face_objs = [face_objs]
+
+    # Select face with largest area
+    def get_area(face):
+        fa = face.get("facial_area", {})
+        if "w" in fa and "h" in fa:
+            return fa["w"] * fa["h"]
+        elif {"x", "x2", "y", "y2"}.issubset(fa):
+            return (fa["x2"] - fa["x"]) * (fa["y2"] - fa["y"])
+        return 0
+
+    largest_face = max(face_objs, key=get_area)
+    facial_area = largest_face.get("facial_area", {})
+    confidence = largest_face.get("face_confidence", 0.0)
+    embedding = np.array(largest_face["embedding"], dtype=np.float32)
+
+    # Compute w, h from bounding box
+    w, h = 0, 0
+    if "w" in facial_area and "h" in facial_area:
+        w, h = facial_area["w"], facial_area["h"]
+    elif {"x", "x2", "y", "y2"}.issubset(facial_area): #other libraries, models returns x2,y2 instead of w,h
+        w = facial_area["x2"] - facial_area["x"]
+        h = facial_area["y2"] - facial_area["y"]
+
+    # Country-specific rejection
+    if country.upper() == "SDN":
+        if w < 40 or h < 50:
+            logging.warning(f"Rejected: SDN face too small ({w}x{h}), minimum 40x50")
+            return [], []
+
+    if confidence < 0.95:
+        logging.warning(f"Rejected: low face detection confidence ({confidence:.3f})")
+        return [], []
+
+    facial_area_filtered = {}
+    x = facial_area.get("x", 0)
+    y = facial_area.get("y", 0)
+    w = facial_area.get("w", 0)
+    h = facial_area.get("h", 0)
+
+    facial_area_filtered = [(x,y,w,h)]
+    return facial_area_filtered, [embedding]
+
+
+def cosine_similarity(vec1, vec2):
+    # vec1 = np.array(vec1)
+    # vec2 = np.array(vec2)
+    #
+    # if vec1.shape != vec2.shape:
+    #     raise ValueError(f"Shape mismatch: {vec1.shape} vs {vec2.shape}")
+    #
+    # similarity = dot(vec1, vec2) / (norm(vec1) * norm(vec2))
+    # return round(float(similarity), 4)
+    similarity = 1 - cosine(vec1, vec2)
+    similarity = round(similarity, 5)
+    similarity = min(1, similarity)
+    return similarity
+
+def extract_face_and_compute_cosine_similarity(selfie, front_face_locations, front_face_encodings):
+        try:
+            if selfie is None:
+                print("Error: Selfie image is None")
+                return 0
+
+            # Ensure the input array is contiguous and in the correct format
+            if not selfie.flags['C_CONTIGUOUS']:
+                selfie = np.ascontiguousarray(selfie)
+
+            # Convert array to uint8 if needed
+            if selfie.dtype != np.uint8:
+                if selfie.max() > 255:
+                    selfie = (selfie / 256).astype(np.uint8)
+                else:
+                    selfie = selfie.astype(np.uint8)
+
+            # Try DeepFace first as it's generally more reliable
+            # start_time = time.time()
+            face_locations1, face_encodings1 = get_facial_encodings_deepface_irq(selfie, country='IRQ')
+            # end_time = time.time()
+
+            if not face_locations1 or not face_encodings1:
+                print("No face detected in Selfie Video by DeepFace")
+                return 0
+
+            # print(f"Face detection took {end_time - start_time:.3f} seconds")
+            face_locations2, face_encodings2 = front_face_locations, front_face_encodings
+
+            if not face_encodings2.any():
+                print('No face detected in front ID')
+                return 0
+
+            largest_face_index1 = face_locations1.index(
+                max(face_locations1, key=lambda loc: (loc[2] - loc[0]) * (loc[3] - loc[1])))
+            largest_face_index2 = face_locations2.index(
+                max(face_locations2, key=lambda loc: (loc[2] - loc[0]) * (loc[3] - loc[1])))
+
+            face_encoding1 = face_encodings1[largest_face_index1]
+            face_encoding2 = face_encodings2[largest_face_index2]
+
+            similarity_score = cosine_similarity(face_encoding1, face_encoding2)
+            # print(f"Calculated similarity score: {similarity_score}")
+
+            return similarity_score
+
+        except Exception as e:
+            print(f"Error in extract_face_and_compute_similarity: {e}")
+            import traceback
+            traceback.print_exc()
+            return 0
+
+def load_and_process_image_deepface_serial(image_input, country=None):
+    """
+    Like load_and_process_image_deepface, but processes angles sequentially (0, 90, 180, 270).
+    Returns face_locations, face_encodings if found, else empty lists.
+    """
+    DeepFace = get_deepface()  # Only load when needed
+    face_recognition = get_face_recognition()  # Only load when needed
+    def process_angle(img, angle):
+        try:
+            if angle != 0:
+                with np.errstate(all='ignore'):
+                    img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
+                    img_pil = Image.fromarray(img_rgb)
+                    rotated = np.ascontiguousarray(img_pil.rotate(angle, expand=True))
+                    img_to_process = cv2.cvtColor(rotated, cv2.COLOR_RGB2BGR)
+                    del img_rgb, img_pil, rotated
+            else:
+                img_to_process = img
+            face_objs = DeepFace.extract_faces(
+                img_to_process,
+                detector_backend='fastmtcnn',
+                enforce_detection=False,
+                align=True
+            )
+            if face_objs and len(face_objs) > 0:
+                confidence = face_objs[0].get('confidence', 0)
+                if confidence < 1:
+                    facial_area = face_objs[0]['facial_area']
+                    if country == 'SDN' and (facial_area['w'] < 40 or facial_area['h'] < 50):
+                        return None, None, 0
+                    elif country != 'SDN' and (facial_area['w'] < 80 or facial_area['h'] < 90):
+                        return None, None, 0
+                if confidence < 0.95:
+                    return None, None, 0
+                return face_objs, img_to_process, confidence
+            del img_to_process
+            return None, None, 0
+        except Exception as e:
+            return None, None, 0
+        finally:
+            if 'img_to_process' in locals():
+                del img_to_process
+    try:
+        if isinstance(image_input, np.ndarray):
+            image = np.ascontiguousarray(image_input)
+            if image.dtype != np.uint8:
+                image = image.astype(np.uint8, copy=False)
+        elif isinstance(image_input, str):
+            image_data = base64.b64decode(image_input)
+            image = cv2.imdecode(np.frombuffer(image_data, np.uint8), cv2.IMREAD_COLOR)
+            del image_data
+        else:
+            return [], []
+        if image is None or image.size == 0:
+            return [], []
+        CONFIDENCE_THRESHOLD = 0.97
+        angles = [0, 90, 180, 270]
+        for angle in angles:
+            face_objs, processed_image, confidence = process_angle(image, angle)
+            if face_objs is not None and confidence >= CONFIDENCE_THRESHOLD:
+                try:
+                    biggest_face = max(face_objs, key=lambda face: face['facial_area']['w'] * face['facial_area']['h'])
+                    facial_area = biggest_face['facial_area']
+                    if confidence < 1:
+                        if country == 'SDN' and (facial_area['w'] < 40 or facial_area['h'] < 50):
+                            return [], []
+                        elif country != 'SDN' and (facial_area['w'] < 80 or facial_area['h'] < 90):
+                            return [], []
+                    x, y, w, h = facial_area['x'], facial_area['y'], facial_area['w'], facial_area['h']
+                    image_rgb = cv2.cvtColor(processed_image, cv2.COLOR_BGR2RGB)
+                    face_locations = [(y, x + w, y + h, x)]
+                    face_encodings = face_recognition.face_encodings(image_rgb, face_locations)
+                    if face_encodings:
+                        return face_locations, face_encodings
+                finally:
+                    if 'processed_image' in locals():
+                        del processed_image
+                    if 'image_rgb' in locals():
+                        del image_rgb
+                    if 'face_objs' in locals():
+                        del face_objs
+        return [], []
+    except Exception as e:
+        return [], []
+    finally:
+        if 'image' in locals():
+            del image
