@@ -122,6 +122,7 @@ def sanity_check_irq_passport(full_name='',
         issue_date = (expiry_date_obj - relativedelta(years=8) + timedelta(days=1)).strftime("%Y-%m-%d")
 
         dob = sanity_utils.convert_dob_to_standard(dob)
+        expiry_date = sanity_utils.convert_dob_to_standard(expiry_date)
 
 
         if gender_letter.lower() not in ['m','f','male','female']:
@@ -253,6 +254,9 @@ def sanity_check_irq_front(
 
 
 
+
+
+
         if not last_name or not last_name_en:
             name = result_dict.get('first_name', '') + " " + result_dict.get('father_name', '')
             name_en = result_dict.get('first_name_en', '') + " " + result_dict.get('father_name_en', '')
@@ -264,7 +268,6 @@ def sanity_check_irq_front(
         if missing:
             return {'error': 'covered_photo', 'error_details': f'Missing or empty fields: {", ".join(missing)}'}
 
-
         result =  {
             "error": "",
             "error_details": "",
@@ -373,6 +376,9 @@ def sanity_check_irq_back(
                 'error_details': f'Missing or empty fields: {", ".join(missing)}'
             }
 
+        dob = sanity_utils.normalise_dates_for_nfc(dob)
+        expiry_date = sanity_utils.normalise_dates_for_nfc(expiry_date)
+
         result =  {
             "error": "",
             "error_details": "",
