@@ -1143,7 +1143,7 @@ def visual_authenticity_check(data, front_id_text, back_id_text, selfie, facial_
 
     if np.any(selfie):
         ## if facial similarity is matching the threshold or even if facial similarity comes above 40 then we say face was detected - approach can be changed
-        if facial_similarity >= face_match_threshold:
+        if facial_similarity > face_match_threshold:
             visual_authenticity['breakdown']['face_detection'] = 'clear'
             visual_authenticity['breakdown']['security_features'] = 'clear'
         else:
@@ -1425,7 +1425,21 @@ def main_details(data, country):
                     }
                 
             main_properties['document_numbers'].append(id_data_t) 
-            main_properties['document_numbers'].append(id_data_v) 
+            main_properties['document_numbers'].append(id_data_v)
+
+        elif data.get('passport_number', None):
+            id_data_t = {
+                "type": "type",
+                "value": "personal_number"
+            }
+            id_data_v = {
+                "type": "value",
+                "value": data['passport_number']
+            }
+
+            main_properties['document_numbers'].append(id_data_t)
+            main_properties['document_numbers'].append(id_data_v)
+
 
         else:
             id_data_t = {
@@ -1439,7 +1453,20 @@ def main_details(data, country):
                 
             main_properties['document_numbers'].append(id_data_t) 
             main_properties['document_numbers'].append(id_data_v)
-        
+
+        if country == 'JOR':
+            id_data_t = {
+                "type": "type",
+                "value": "personal_number"
+            }
+            id_data_v = {
+                "type": "value",
+                "value": data.get('passport_number','')
+            }
+
+            main_properties['document_numbers'].append(id_data_t)
+            main_properties['document_numbers'].append(id_data_v)
+
         # main_properties['document_type'] = 'national_identity_card'
         # main_properties['name'] = data.get('name')
         # main_properties['first_name'] = data.get('first_name', '')
@@ -1506,7 +1533,7 @@ def main_details(data, country):
                 main_prop_data = {}
                 keys_to_remove = ['name', 'occupation', 'employer', 'mrz_line3', 'family_sponsor']
                 if data.get('doc_type') == 'passport':
-                    keys = ['full_name', 'first_name', 'last_name', 'father_name', 'mother_name', 'place_of_birth', 'dob', 'issuing_date', 'nationality', 'gender', 'place_of_issue', 'passport_number', 'mrz1', 'mrz2', 'mrz', 'issuing_country']
+                    keys = ['full_name', 'first_name', 'last_name', 'father_name', 'mother_name', 'place_of_birth', 'dob', 'issuing_date', 'nationality', 'gender', 'place_of_issue', 'passport_national_number', 'mrz1', 'mrz2', 'mrz', 'issuing_country']
 
             if country == 'PSE':
                 main_prop_data = {}
@@ -1718,11 +1745,11 @@ def form_final_facial_similarity_report(data, selfie, facial_similarity, livenes
             facial_report['breakdown']['image_integrity']['breakdown']['face_detected']['result'] = ''
             facial_report['breakdown']['image_integrity']['breakdown']['source_integrity']['result'] = ''
 
-        if np.any(selfie) and facial_similarity >= face_match_threshold:
+        if np.any(selfie) and facial_similarity > face_match_threshold:
             facial_report['breakdown']['face_comparison']['breakdown']['face_match']['result'] = 'clear'
             facial_report['breakdown']['face_comparison']['result'] = 'clear'
             facial_report['breakdown']['image_integrity']['breakdown']['face_detected']['result'] = 'clear'
-        elif facial_similarity != 0 and facial_similarity < face_match_threshold:
+        elif facial_similarity != 0 and facial_similarity <= face_match_threshold:
             facial_report['breakdown']['face_comparison']['breakdown']['face_match']['result']  = 'consider'
             facial_report['breakdown']['face_comparison']['result'] = 'consider'
             facial_report['breakdown']['image_integrity']['breakdown']['face_detected']['result'] = 'clear'
@@ -1751,7 +1778,7 @@ def form_final_facial_similarity_report(data, selfie, facial_similarity, livenes
             facial_report['breakdown']['visual_authenticity']['breakdown']['liveness_detected']['result'] = 'consider'
             facial_report['breakdown']['visual_authenticity']['breakdown']['spoofing_detection']['result'] = 'consider'
             facial_report['breakdown']['visual_authenticity']['result'] = 'consider'
-        elif liveness_result == 'clear' and facial_similarity < face_match_threshold:
+        elif liveness_result == 'clear' and facial_similarity <= face_match_threshold:
             facial_report['breakdown']['visual_authenticity']['breakdown']['spoofing_detection']['result'] = 'consider'
             facial_report['breakdown']['visual_authenticity']['result'] = 'consider'
         elif liveness_result == None:
